@@ -1,6 +1,6 @@
 /* top_nls.c - provide the basis for future nls translations */
 /*
- * Copyright (c) 2011-2012,     by: James C. Warner
+ * Copyright (c) 2011-2013,     by: James C. Warner
  *    All rights reserved.      8921 Hilloway Road
  *                              Eden Prairie, Minnesota 55347 USA
  *
@@ -19,9 +19,6 @@
 
 #include <locale.h>
 #include <stdio.h>
-#ifdef VALIDATE_NLS
-#include <stdlib.h>
-#endif
 #include <string.h>
 
 #include "../include/nls.h"
@@ -29,6 +26,10 @@
 #include "top.h"
 #include "top_nls.h"
 
+#ifdef VALIDATE_NLS
+#include <stdlib.h>
+#endif
+
         // Programmer Note(s):
         //  Preparation ---------------------------------------------
         //    Unless you have *something* following the gettext macro,
@@ -66,9 +67,10 @@
         //                           with "... charset=UTF-8\n"
         //        . translate msgstr values, leaving msgid unchanged
         //    msgfmt ll_CC.po --strict --output-file=procps-ng.mo
-        //     . after which chmod 644
-        //     . move to /usr/share/local/ll_CC/LC_MESSAGES/ directory
-        //
+        //     . after which ensure, chmod 644
+        //     . then copy
+        //         to /usr/share/locale-langpack/ll_CC/LC_MESSAGES/
+        //         or /usr/share/locale/ll_CC/LC_MESSAGES/
         //  Testing -------------------------------------------------
         //    export LC_ALL= && export LANGUAGE=ll_CC
         //    run some capable program like top
@@ -254,6 +256,27 @@ static void build_two_nlstabs (void) {
 /* Translation Hint: maximum 'vMn' = 3 */
    Head_nlstab[P_FV2] = _("vMn");
    Desc_nlstab[P_FV2] = _("Minor Faults delta");
+/* Translation Hint: maximum 'USED' = 4 */
+   Head_nlstab[P_USE] = _("USED");
+   Desc_nlstab[P_USE] = _("Res+Swap Size (KiB)");
+/* Translation Hint: maximum 'nsIPC' = 10 */
+   Head_nlstab[P_NS1] = _("nsIPC");
+   Desc_nlstab[P_NS1] = _("IPC namespace Inode");
+/* Translation Hint: maximum 'nsMNT' = 10 */
+   Head_nlstab[P_NS2] = _("nsMNT");
+   Desc_nlstab[P_NS2] = _("MNT namespace Inode");
+/* Translation Hint: maximum 'nsNET' = 10 */
+   Head_nlstab[P_NS3] = _("nsNET");
+   Desc_nlstab[P_NS3] = _("NET namespace Inode");
+/* Translation Hint: maximum 'nsPID' = 10 */
+   Head_nlstab[P_NS4] = _("nsPID");
+   Desc_nlstab[P_NS4] = _("PID namespace Inode");
+/* Translation Hint: maximum 'nsUSER' = 10 */
+   Head_nlstab[P_NS5] = _("nsUSER");
+   Desc_nlstab[P_NS5] = _("USER namespace Inode");
+/* Translation Hint: maximum 'nsUTS' = 10 */
+   Head_nlstab[P_NS6] = _("nsUTS");
+   Desc_nlstab[P_NS6] = _("UTS namespace Inode");
 }
 
 
@@ -273,15 +296,15 @@ static void build_norm_nlstab (void) {
    .  c-code logic changes.
    . */
 
-   Norm_nlstab[EXIT_signals_fmt] = _("\n"
+   Norm_nlstab[EXIT_signals_fmt] = _(""
       "\tsignal %d (%s) was caught by %s, please\n"
       "\tsee http://www.debian.org/Bugs/Reporting\n");
    Norm_nlstab[WRONG_switch_fmt] = _(""
       "inappropriate '%s'\n"
-      "usage:\t%s%s");
+      "Usage:\n  %s%s");
    Norm_nlstab[HELP_cmdline_fmt] = _(""
-      "\t%s\n"
-      "usage:\t%s%s");
+      "  %s\n"
+      "Usage:\n  %s%s");
    Norm_nlstab[FAIL_statopn_fmt] = _("failed /proc/stat open: %s");
    Norm_nlstab[FAIL_openlib_fmt] = _("failed openproc: %s");
    Norm_nlstab[BAD_delayint_fmt] = _("bad delay interval '%s'");
@@ -292,19 +315,18 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[BAD_widtharg_fmt] = _("bad width arg '%s', must > %d");
    Norm_nlstab[UNKNOWN_opts_fmt] = _(""
       "unknown option '%c'\n"
-      "usage:\t%s%s");
+      "Usage:\n  %s%s");
    Norm_nlstab[DELAY_secure_txt] = _("-d disallowed in \"secure\" mode");
    Norm_nlstab[DELAY_badarg_txt] = _("-d requires positive argument");
    Norm_nlstab[ON_word_only_txt] = _("On");
    Norm_nlstab[OFF_one_word_txt] = _("Off");
 /* Translation Hint: Only the following words should be translated
-   .                 delay, limit, user, cols (abbreviation for columns)*/
-   Norm_nlstab[USAGE_abbrev_txt] = _(" -hv | -bcHiSs -d delay -n limit -u|U user -p pid[,pid] -w [cols]");
+   .                 secs (seconds), max (maximum), user, field, cols (columns)*/
+   Norm_nlstab[USAGE_abbrev_txt] = _(" -hv | -bcHiOSs -d secs -n max -u|U user -p pid(s) -o field -w [cols]");
    Norm_nlstab[FAIL_statget_txt] = _("failed /proc/stat read");
    Norm_nlstab[FOREST_modes_fmt] = _("Forest mode %s");
    Norm_nlstab[FAIL_tty_get_txt] = _("failed tty get");
-   Norm_nlstab[FAIL_tty_mod_fmt] = _("failed Tty_tweaked set: %s");
-   Norm_nlstab[FAIL_tty_raw_fmt] = _("failed Tty_raw set: %s");
+   Norm_nlstab[FAIL_tty_set_fmt] = _("failed tty set: %s");
    Norm_nlstab[CHOOSE_group_txt] = _("Choose field group (1 - 4)");
    Norm_nlstab[DISABLED_cmd_txt] = _("Command disabled, 'A' mode required");
    Norm_nlstab[DISABLED_win_fmt] = _("Command disabled, activate %s with '-' or '_'");
@@ -314,11 +336,11 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[DELAY_change_fmt] = _("Change delay from %.1f to");
    Norm_nlstab[THREADS_show_fmt] = _("Show threads %s");
    Norm_nlstab[IRIX_curmode_fmt] = _("Irix mode %s");
-   Norm_nlstab[GET_pid2kill_txt] = _("pid to signal/kill");
+   Norm_nlstab[GET_pid2kill_fmt] = _("PID to signal/kill [default pid = %d]");
    Norm_nlstab[GET_sigs_num_fmt] = _("Send pid %d signal [%d/sigterm]");
    Norm_nlstab[FAIL_signals_fmt] = _("Failed signal pid '%d' with '%d': %s");
    Norm_nlstab[BAD_signalid_txt] = _("Invalid signal");
-   Norm_nlstab[GET_pid2nice_txt] = _("PID to renice");
+   Norm_nlstab[GET_pid2nice_fmt] = _("PID to renice [default pid = %d]");
    Norm_nlstab[GET_nice_num_fmt] = _("Renice PID %d to value");
    Norm_nlstab[FAIL_re_nice_fmt] = _("Failed renice of PID %d to %d: %s");
    Norm_nlstab[NAME_windows_fmt] = _("Rename window '%s' to (1-3 chars)");
@@ -333,8 +355,6 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[FAIL_alloc_r_txt] = _("failed memory re-allocate");
    Norm_nlstab[BAD_numfloat_txt] = _("Unacceptable floating point");
    Norm_nlstab[BAD_username_txt] = _("Invalid user");
-   Norm_nlstab[FAIL_sigstop_fmt] = _("failed sigprocmask, SIG_BLOCK: %s");
-   Norm_nlstab[FAIL_sigmask_fmt] = _("failed sigprocmask, SIG_SETMASK: %s");
    Norm_nlstab[FOREST_views_txt] = _("forest view");
    Norm_nlstab[FAIL_widepid_txt] = _("failed pid maximum size test");
    Norm_nlstab[FAIL_widecpu_txt] = _("failed number of cpus test");
@@ -353,6 +373,15 @@ static void build_norm_nlstab (void) {
 /* Translation Hint: This is an abbreviation (limit 3 characters) for:
    .                 gibibytes (1,073,741,824 bytes) */
    Norm_nlstab[AMT_gigabyte_txt] = _("GiB");
+/* Translation Hint: This is an abbreviation (limit 3 characters) for:
+   .                 tebibytes (1,099,511,627,776 bytes) */
+   Norm_nlstab[AMT_terabyte_txt] = _("TiB");
+/* Translation Hint: This is an abbreviation (limit 3 characters) for:
+   .                 pebibytes (1,024 tebibytes) */
+   Norm_nlstab[AMT_petabyte_txt] = _("PiB");
+/* Translation Hint: This is an abbreviation (limit 3 characters) for:
+   .                 exbibytes (1,024 pebibytes) */
+   Norm_nlstab[AMT_exxabyte_txt] = _("EiB");
    Norm_nlstab[WORD_threads_txt] = _("Threads");
    Norm_nlstab[WORD_process_txt] = _("Tasks");
 /* Translation Hint: The following "word" is meant to represent either a single
@@ -368,9 +397,73 @@ static void build_norm_nlstab (void) {
    Norm_nlstab[GET_find_str_txt] = _("Locate string");
    Norm_nlstab[FIND_no_find_fmt] = _("%s\"%s\" not found");
    Norm_nlstab[XTRA_fixwide_fmt] = _("width incr is %d, change to (0 default, -1 auto)");
-#ifndef WARN_CFG_OFF
-   Norm_nlstab[XTRA_warncfg_txt] = _("Overwrite existing old style rcfile?");
+   Norm_nlstab[XTRA_warncfg_txt] = _("Overwrite existing obsolete/corrupted rcfile?");
+   Norm_nlstab[XTRA_badflds_fmt] = _("unrecognized field name '%s'");
+   Norm_nlstab[XTRA_winsize_txt] = _("even using field names only, window is now too small");
+#ifndef INSP_OFFDEMO
+   Norm_nlstab[YINSP_demo01_txt] = _("Open Files");
+   Norm_nlstab[YINSP_demo02_txt] = _("NUMA Info");
+   Norm_nlstab[YINSP_demo03_txt] = _("Log");
+   Norm_nlstab[YINSP_deqfmt_txt] = _("the '=' key will eventually show the actual file read or command(s) excuted ...");
+   Norm_nlstab[YINSP_deqtyp_txt] = _("demo");
+   Norm_nlstab[YINSP_dstory_txt] = _(""
+      "This is simulated output representing the contents of some file or the output\n"
+      "from some command.  Exactly which commands and/or files are solely up to you.\n"
+      "\n"
+      "Although this text is for information purposes only, it can still be scrolled\n"
+      "and searched like real output will be.  You are encouraged to experiment with\n"
+      "those features as explained in the prologue above.\n"
+      "\n"
+      "To enable real Inspect functionality, entries must be added to the end of the\n"
+      "top personal personal configuration file.  You could use your favorite editor\n"
+      "to accomplish this, taking care not to disturb existing entries.\n"
+      "\n"
+      "Another way to add entries is illustrated below, but it risks overwriting the\n"
+      "rcfile.  Redirected echoes must not replace (>) but append (>>) to that file.\n"
+      "\n"
+      "  /bin/echo -e \"pipe\\tOpen Files\\tlsof -P -p %d 2>&1\" >> ~/.toprc\n"
+      "  /bin/echo -e \"file\\tNUMA Info\\t/proc/%d/numa_maps\" >> ~/.toprc\n"
+      "  /bin/echo -e \"pipe\\tLog\\ttail -n200 /var/log/syslog | sort -Mr\" >> ~/.toprc\n"
+      "\n"
+      "If you don't know the location or name of the top rcfile, use the 'W' command\n"
+      "and note those details.  After backing up the current rcfile, try issuing the\n"
+      "above echoes exactly as shown, replacing '.toprc' as appropriate.  The safest\n"
+      "approach would be to use copy then paste to avoid any typing mistakes.\n"
+      "\n"
+      "Finally, restart top to reveal what actual Inspect entries combined with this\n"
+      "new command can offer.  The possibilities are endless, especially considering\n"
+      "that 'pipe' type entries can include shell scripts too!\n"
+      "\n"
+      "For additional important information, please consult the top documentation.\n"
+      "Then enhance top with your very own customized 'file' and 'pipe' entries.\n"
+      "\n"
+      "Enjoy!\n");
+   Norm_nlstab[YINSP_noents_txt] = _("to enable 'Y' press <Enter> then type 'W' and restart top");
+#else
+   Norm_nlstab[YINSP_noents_txt] = _("to enable 'Y' please consult the top man page (press Enter)");
 #endif
+   Norm_nlstab[YINSP_failed_fmt] = _("Selection failed with: %s\n");
+   Norm_nlstab[YINSP_pidbad_fmt] = _("unable to inspect, pid %d not found");
+   Norm_nlstab[YINSP_pidsee_fmt] = _("inspect at PID [default pid = %d]");
+   Norm_nlstab[YINSP_rcfile_fmt] = _("could not parse rcfile inspect entry %d");
+   Norm_nlstab[YINSP_status_fmt] = _("%s: %*d-%-*d lines, %*d-%*d columns, %lu bytes read");
+   Norm_nlstab[YINSP_workin_txt] = _("patience please, working...");
+/* Translation Hint: Below are 2 abbreviations which can be as long as needed:
+   .                 FLD = FIELD, VAL = VALUE */
+   Norm_nlstab[OSEL_prompts_fmt] = _("add filter #%d (%s) as: [!]FLD?VAL");
+   Norm_nlstab[OSEL_casenot_txt] = _("ignoring case");
+   Norm_nlstab[OSEL_caseyes_txt] = _("case sensitive");
+   Norm_nlstab[OSEL_errdups_txt] = _("duplicate filter was ignored");
+   Norm_nlstab[OSEL_errdelm_fmt] = _("'%s' filter delimiter is missing");
+   Norm_nlstab[OSEL_errvalu_fmt] = _("'%s' filter value is missing");
+   Norm_nlstab[WORD_include_txt] = _("include");
+   Norm_nlstab[WORD_exclude_txt] = _("exclude");
+   Norm_nlstab[OSEL_statlin_fmt] = _("<Enter> to resume, filters: %s");
+   Norm_nlstab[WORD_noneone_txt] = _("none");
+   Norm_nlstab[NUMA_nodenam_fmt] = _("Node%-2d:");
+   Norm_nlstab[NUMA_nodeget_fmt] = _("expand which node (0-%d)");
+   Norm_nlstab[NUMA_nodebad_txt] = _("invalid node");
+   Norm_nlstab[NUMA_nodenot_txt] = _("sorry, NUMA extensions unavailable");
 }
 
 
@@ -400,7 +493,7 @@ static void build_uniq_nlstab (void) {
    .  adhere to that goal lest the translated text be truncated.
    .
    .  If you would like additional information regarding these strings,
-   .  please see the prolog to the show_special function in the top.c
+   .  please see the prologue to the show_special function in the top.c
    .  source file.
    . */
 
@@ -408,26 +501,26 @@ static void build_uniq_nlstab (void) {
       "Help for Interactive Commands~2 - %s\n"
       "Window ~1%s~6: ~1Cumulative mode ~3%s~2.  ~1System~6: ~1Delay ~3%.1f secs~2; ~1Secure mode ~3%s~2.\n"
       "\n"
-      "  Z~5,~1B~5       Global: '~1Z~2' change color mappings; '~1B~2' disable/enable bold\n"
-      "  l,t,m     Toggle Summaries: '~1l~2' load avg; '~1t~2' task/cpu stats; '~1m~2' mem info\n"
-      "  1,I       Toggle SMP view: '~11~2' single/separate states; '~1I~2' Irix/Solaris mode\n"
+      "  Z~5,~1B~5,E,e   Global: '~1Z~2' colors; '~1B~2' bold; '~1E~2'/'~1e~2' summary/task memory scale\n"
+      "  l,t,m     Toggle Summary: '~1l~2' load avg; '~1t~2' task/cpu stats; '~1m~2' memory info\n"
+      "  0,1,2,3,I Toggle: '~10~2' zeros; '~11~2/~12~2/~13~2' cpus or numa node views; '~1I~2' Irix mode\n"
       "  f,F,X     Fields: '~1f~2'/'~1F~2' add/remove/order/sort; '~1X~2' increase fixed-width\n"
       "\n"
-      "  L,&,<,> . Locate: '~1L~2'/'~1&~2' find/again; Move sort column: '~1<~2'/'~1>~2' left/right\n" \
+      "  L,&,<,> . Locate: '~1L~2'/'~1&~2' find/again; Move sort column: '~1<~2'/'~1>~2' left/right\n"
       "  R,H,V,J . Toggle: '~1R~2' Sort; '~1H~2' Threads; '~1V~2' Forest view; '~1J~2' Num justify\n"
       "  c,i,S,j . Toggle: '~1c~2' Cmd name/line; '~1i~2' Idle; '~1S~2' Time; '~1j~2' Str justify\n"
       "  x~5,~1y~5     . Toggle highlights: '~1x~2' sort field; '~1y~2' running tasks\n"
       "  z~5,~1b~5     . Toggle: '~1z~2' color/mono; '~1b~2' bold/reverse (only if 'x' or 'y')\n"
-      "  u,U     . Show: '~1u~2' effective user; '~1U~2' real, saved, file or effective user\n"
-      "  n or #  . Set maximum tasks displayed\n"
-      "  C,...   . Toggle scroll coordinates msg for: ~1up~2,~1down~2,~1left~2,right~2,~1home~2,~1end~2\n"
+      "  u,U,o,O . Filter by: '~1u~2'/'~1U~2' effective/any user; '~1o~2'/'~1O~2' other criteria\n"
+      "  n,#,^O  . Set: '~1n~2'/'~1#~2' max tasks displayed; Show: ~1Ctrl~2+'~1O~2' other filter(s)\n"
+      "  C,...   . Toggle scroll coordinates msg for: ~1up~2,~1down~2,~1left~2,~1right~2,~1home~2,~1end~2\n"
       "\n"
       "%s"
-      "  W         Write configuration file\n"
+      "  W,Y       Write configuration file '~1W~2'; Inspect other output '~1Y~2'\n"
       "  q         Quit\n"
       "          ( commands shown with '.' require a ~1visible~2 task display ~1window~2 ) \n"
       "Press '~1h~2' or '~1?~2' for help with ~1Windows~2,\n"
-      "any other key to continue ");
+      "Type 'q' or <Esc> to continue ");
 
 /* Translation Hint: As is true for the text above, the "keys" shown to the left and
    .                 also imbedded in the translatable text (along with escape seqs)
@@ -501,21 +594,21 @@ static void build_uniq_nlstab (void) {
       "   available toggles: ~1B~2 =disable bold globally (~1%s~2),\n"
       "       ~1z~2 =color/mono (~1%s~2), ~1b~2 =tasks \"bold\"/reverse (~1%s~2)\n"
       "\n"
-      "Select ~1target~2 as upper case letter:\n"
+      "1) Select a ~1target~2 as an upper case letter, ~1current target~2 is ~1 %c ~4:\n"
       "   S~2 = Summary Data,~1  M~2 = Messages/Prompts,\n"
       "   H~2 = Column Heads,~1  T~2 = Task Information\n"
-      "Select ~1color~2 as number:\n"
+      "2) Select a ~1color~2 as a number, ~1current color~2 is ~1 %d ~4:\n"
       "   0~2 = black,~1  1~2 = red,    ~1  2~2 = green,~1  3~2 = yellow,\n"
       "   4~2 = blue, ~1  5~2 = magenta,~1  6~2 = cyan, ~1  7~2 = white\n"
       "\n"
-      "Selected: ~1target~2 ~1 %c ~4; ~1color~2 ~1 %d ~4\n"
-      "   press 'q' to abort changes to window '~1%s~2'\n"
-      "   press 'a' or 'w' to commit & change another, <Enter> to commit and end ");
+      "3) Then use these keys when finished:\n"
+      "   'q' to abort changes to window '~1%s~2'\n"
+      "   'a' or 'w' to commit & change another, <Enter> to commit and end ");
 
    Uniq_nlstab[FIELD_header_fmt] = _(""
       "Fields Management~2 for window ~1%s~6, whose current sort field is ~1%s~2\n"
       "   Navigate with Up/Dn, Right selects for move then <Enter> or Left commits,\n"
-      "   'd' or <Space> toggles display, 's' sets sort.  Use 'q' or <Esc> to end! ");
+      "   'd' or <Space> toggles display, 's' sets sort.  Use 'q' or <Esc> to end!\n");
 
    Uniq_nlstab[STATE_line_1_fmt] = _("%s:~3"
       " %3u ~2total,~3 %3u ~2running,~3 %3u ~2sleeping,~3 %3u ~2stopped,~3 %3u ~2zombie~3\n");
@@ -542,8 +635,18 @@ static void build_uniq_nlstab (void) {
    .                 abbreviations: Mem = physical memory/ram, Swap = the linux swap file
    .                 words:         total, used, free, buffers, cached */
    Uniq_nlstab[MEMORY_lines_fmt] = _(""
-      "%s Mem: ~3 %8lu ~2total,~3 %8lu ~2used,~3 %8lu ~2free,~3 %8lu ~2buffers~3\n"
-      "%s Swap:~3 %8lu ~2total,~3 %8lu ~2used,~3 %8lu ~2free,~3 %8lu ~2cached~3\n");
+      "%s Mem: ~3 %9.9s~2total,~3 %9.9s~2used,~3 %9.9s~2free,~3 %9.9s~2buffers~3\n"
+      "%s Swap:~3 %9.9s~2total,~3 %9.9s~2used,~3 %9.9s~2free.~3 %9.9s~2cached Mem~3\n");
+
+   Uniq_nlstab[INSP_hdrsels_fmt] = _(""
+      "Inspection~2 Pause at: pid ~1%d~6 running ~1%s~6 as user ~1%s~6\n"
+      "Use~2:  left/right then <Enter> to ~1select~5 an option; 'q' or <Esc> to ~1end~5 !\n"
+      "Options~2: ~1%s\n");
+
+   Uniq_nlstab[INSP_hdrview_fmt] = _(""
+      "Inspection~2 View at: ~1%s~3, ~1%s~3, ~1%s~3.  Locating: ~1%s~6\n"
+      "Use~2:  left/right/up/down/etc to ~1navigate~5 the output; 'L'/'&' to ~1locate~5/~1next~5.\n"
+      "Or~2:   <Enter> to ~1select another~5; 'q' or <Esc> to ~1end~5 !\n");
 }
 
 
